{% extends "base.html" %}
{% load kegweblib crispy_forms_tags %}

{% block title %}Reset Password{% endblock %}

{% block content %}
<h2>Reset Password</h2>
<form action="" method="POST">{% csrf_token %}
  {{ form.as_p }}
  <input type="submit" class="btn btn-success" name="submit" value="Send E-Mail" />
</form>
{% endblock %}
